/*****************************************************************************************************************************      
 * Markus Otte  - www.markusotte.de  (C)2014/2015                                                                                          *
 * Projekt: Luftwärmetauscher Stand V9.0                                                                                     *
 * Beschreibung:  Temperaturwerte werden von jeweils zwei DS1820 Temp.sensoren und zwei DHT Feuchte/Temp an ein php-Script   *
 * gesendet, welches diese Daten in eine MySQL-Datenbank schreibt.                                                           *
 *                                                                                                                           *
 * Einige Programmteile wurden übernommen von www.fluuux.de, www.macherzin.net und Arduino.cc                                *                                                                                                                          *
 *                                                                                                                           *
 *                                                                                                                           *                                                                                                                          *
 * Folgende Werte müssen angepasst werden:                                                                                   *
 * --------------------------------------------------------------------------------------------------------------------------*
 * WebIntervall = Zeitraum in welchen Abständen die Temperaturdaten vom Arduino zur Datenbank übertragen werden sollen,      *
 * mac[] = MAC-Adresse des Ethernet Shields                                                                                  *
 * ip[] = IP-Adresse über die der Arduino erreichbar sein soll.                                                              *
 * server[] = Name der Domain zu der die Verbindung aufgebaut werden soll                                                    *
 * url[] = Pfad und Name der Datei die für den Upload der übergebenen Daten zur Datenbank zuständig ist.                     *
 * key[] = Kennwort dass mit dem Kennwort in der php-Datei übereinstimmen muss (Sicherheitsaspekt)                           *
 *****************************************************************************************************************************/

#include <SPI.h>
#include <Ethernet.h>             // library für Ethernet Funktionen
#include <DallasTemperature.h>    // library für Temperatur Sensor DS1822
#include <OneWire.h>              // library für Onewire Bus
#include <dht.h>                  // library für DHT22
#include <Wire.h>                 // library für Seriellen bus
#include <EthernetUdp.h>          // Über UDP NTP-Zeit aus dem Internet holen
#include <Time.h>

#define PinS1 2            // Schalter 1 an Pin 2
#define PinS2 3            // Schalter 2 an Pin 3
#define PinFanIN 8         // Lüfter Innen an Pin 8
#define PinFanOUT 9        // Lüfter Aussen an Pin 9
#define DHT1_PIN 5         // Zur Messung DHT-Sensor 1 verwendeter Pin 4 (Innen)
#define DHT2_PIN 6         // Zur Messung DHT-Sensor 2 verwendeter Pin 5 (Aussen)
#define Hmax 90            // Luftfeuchte bei 12 Grad
#define dH -4.2857         // Negative Steigung(Delta) Feuchte bei Temperaturerhöhung
#define Tmin 12            // Minimale Temperatur 12 Grad
#define TIME_HEADER  "T"   // Header tag for serial time sync message
#define TIME_REQUEST  7    // ASCII bell character requests a time sync message 
#define aTp 17.271         // fuer Formel Taupunkt
#define bTp 237.7          // fuer Formel Taupunkt
#define OS1 0.16           // Offset Temp Sensor 1 (alle Offsets bitte mit allen Temp.Sensoren abgleichen!)
#define OS2 0.50           // Offset Temp Sensor 2
#define OS3 -0.30          // Offset Temp Sensor 3
#define OS4 0.0            // Offset Temp Sensor 4
#define OSH3 0.0           // Offset Feuchte Sensor 3
#define OSH4 0.0           // Offset Feuchte Sensor 4

dht DHT2;   // Initialisieren der DHTs
dht DHT1;   // Initialisieren der DHTs

boolean S1; // Schalter S1
boolean S2; // Schalter S2
  
OneWire  ds(7);                          // pin für DS1820 Temperatursensoren

//DeviceAdressen der einzelnen ds1820 Temperatursensoren.
DeviceAddress sensor1 = { 
  0x10, 0xCF, 0xEB, 0x5F, 0x2, 0x8, 0x0, 0x91 };
DeviceAddress sensor2 = { 
  0x10, 0x3E, 0xF7, 0x5F, 0x2, 0x8, 0x0, 0x89 };
// **************************************************************************************
//ETHERNET-SETTINGS
byte mac[]     = { 
  0x00, 0xAA, 0xBB, 0xCC, 0xDE, 0x02 };   // MAC-Adresse des Arduino
byte ip[]      = { 
  192, 168, 1, 164 };                     // IP-Adresse des Arduino
byte gateway[] = { 
  192, 168, 1, 1 };                       // Gateway
byte subnet[]  = { 
  255, 255, 255, 0 };                     // SubNet

// Angaben für die NTP Zeitabfrage 
IPAddress timeServer(134, 130, 4, 17);     // rwth.aachen NTP server
unsigned int localPort = 8888;             // lokaler Port für UDP packets
const int NTP_PACKET_SIZE= 48;             // NTP time Paketgröße in den ersten 48 bytes
byte packetBuffer[ NTP_PACKET_SIZE];       //Puffer für ein und ausgehende Pakete 
EthernetUDP Udp; //  UDP Instanz

unsigned long UnixTime = 0;
unsigned long Stunde = 0;
unsigned long Minute = 0;
unsigned long Sekunde = 0;

EthernetClient client;
char server[]    = "markus-otte.de";                      // Domain des Servers
// byte server[]  = {85, 13, 152, 62};                    // alternative Angabe, die IP-Adresse des Servers
char url[]     = "/hierStehtDerPfad/arduino/temperatur/SaveTempToMySQL.php"; // Pfad zur PHP-Datei
char key[]     = "PasswortXYZ";                           // Kennwort aus PHP-Datei
char c;                                                   // Variable für Rückgabe des Servers
char* komment[]={
  "Umluft_AUS","Umluft_EIN", "Hand_EIN", "Deaktiviert"};  // Kommentar für Datenbank - Ohne Leerzeichen !!!!
int kom = 0;                                              // Kommentarauswahl 0 = komment[0]
int IntervalFan1 = 1;                                     // Interval Fan 1
int IntervalFan2 = 1;                                     // Interval Fan 2

DallasTemperature sensors(&ds);                               
int numSensors;                                            // Variable zum Speichern der Anzahl der Temperatur-Sensoren
float temp1;        // Zuluft Temp.
float temp2;        // Abluft Temp.
float dht1_h;       // Innen Feuchte
float dht1_t;       // Innen Temp.
float dht2_h;       // Aussen Feuchte
float dht2_t;       // Aussen Temp.
float SollH;        // Berechnete Sollfeuchte Innenraum
float Taupunkt;     // Berechneter Taupunkt Innen
float taupunktTmp;  // Berechneter Taupunkt Innen

boolean zuluft = false;  // Lüfter Ein
boolean abluft = false;  // Lüfter Ein


// ****** SETUP ***********************************************************

void setup()
{
  pinMode(PinS1,INPUT); // Schalter 1
  digitalWrite(PinS1, HIGH); // Pullupwiderstand aktivieren (gegen Masse schalten)
  pinMode(PinS2,INPUT); // Schalter 2
  digitalWrite(PinS2, HIGH); // Pullupwiderstand aktivieren (gegen Masse schalten)
  pinMode(PinFanIN,OUTPUT); // Lüfter Zuluft
  digitalWrite(PinFanIN, HIGH); // Ausschalten
  pinMode(PinFanOUT,OUTPUT); // Lüfter Abluft
  digitalWrite(PinFanOUT, HIGH); // Ausschalten

  delay(1000);

  Serial.begin(9600);
  Serial.flush();
  delay(200);

  Serial.println("Arduino TemperaturLogger");
  Serial.println("Ethernet initialisieren..."); 

  Ethernet.begin(mac, ip);
  delay(1000);                                                // warten, bis Ethernet gestartet
  Udp.begin(localPort);

  //Anzahl DS1820 Sensoren abfragen
  sensors.begin();                                            // DS1820 Sensoren
  Serial.println("Temperatur-Sensoren ermitteln...");

  numSensors = sensors.getDeviceCount();                      // Anzahl der angeschlossenen Sensoren in numSensors speichern

  if(numSensors > 0)                                          // Es wurde mindestens 1 Sensor gefunden                                       
  {
    Serial.print(numSensors);
    Serial.println( " Temperatur-Sensoren gefunden.");
  }         
  else                                                        // Es wurde kein Sensor gefunden
  {
    Serial.println("Keine Temperatur-Sensoren gefunden.");
  }
}

// ****** Hauptprogramm Loop START ***********************************************************
void loop()
{
  // *****************************************************************************************
  if (Sekunde == 10 ) // ************ Innen und Aussentemperaturen auslesen *
  {
    DS1820auslesen(); 
    DHT22lesen1();
    DHT22lesen2();
    SollH = (Hmax + (dht1_t - Tmin) * dH);  // Ideale Wohlfühlfeuchte berechnen
    taupunktTmp = ((aTp * dht1_t) / (bTp + dht1_t)) + log(dht1_h / 100);
    Taupunkt = (bTp * taupunktTmp) / (aTp - taupunktTmp);
  }
  // ***************************************************************************************** 
  if ((Sekunde == 20 && Minute == 0)|| (Sekunde == 20 && Minute == 20)|| (Sekunde == 20 && Minute == 40))
  {
    NTPServer(); // Zeitserver auslesen
    ZeitSync();  // Zeit mit Arduino synchronisieren
    Serial.print(day());
    Serial.print(" ");
    Serial.print(month());
    Serial.print(" ");
    Serial.print(year()); 
    Serial.println(); 
  }
  // ***************************************************************************************** 
  if (Sekunde == 55 && Minute == 59) WebServer(); // Zum Webserver verbinden 
  // *****************************************************************************************
  kom = 0;  // Kommentar = 0   (Umluft AUS)
  S1 = digitalRead(PinS1);  // Schalter 1 Links = Automatik EIN/AUS   AUS=HIGH
  S2 = digitalRead(PinS2);  // Schalter 2 Rechts = HAND EIN           AUS=HIGH
  if (S1 == LOW && (S2 == HIGH)) Automatik(); // S1 = EIN und S2 = AUS
  if (S2 == LOW) Handbetrieb();
  if ((S1 == HIGH) &&  (S2 == HIGH)) // Beide Schalter AUS
  {
    digitalWrite(PinFanIN,HIGH); // Ausschalten
    digitalWrite(PinFanOUT,HIGH); // Ausschalten    
    kom = 3;    // Kommentar = 0   (Deaktiviert)
  }
  // ********************************* Zeitzähler  *******************************************  
  ZeitZaehler();
  // ********************************* StrG+Hoch+M  ******************************************
  Monitor();  // Ausgabe Serieller Monitor  
  // ********************************* Pause  ************************************************ 
  delay(800);  // Zeitanpassung eine Sekunde
}

// ****** Hauptprogramm Loop ENDE ***********************************************************

void WebServer()
{
  Daten_senden(temp1, temp2, dht1_t, dht2_t, Taupunkt, dht1_h, dht2_h, SollH, zuluft, abluft, komment[kom]);                                 // Temperaturdaten an Server übertragen

  delay(700);

  byte maxReads = 10; //Seconds
  while ((maxReads-- > 0) && client.connected())              // Antwort des Servers lesen
  {
    delay(1000);
    while (client.available())
    {
      char response = client.read();
      Serial.print(response);
    }
  }
  client.stop();
  Serial.println("Done.");
  client.flush();
}


/******************************
 * Funktion
 * Daten an Server senden   *
 *
 *******************************/
void Daten_senden(float T1, float T2, float T3, float T4, float TP, float F3, float F4, float FX, boolean ZU, boolean AB, char KO[19])
{
  if (client.connect(server, 80)) // Verbindung zum Server aufbauen
    {
    Serial.println("Verbunden, Sende Daten...");
    client.print("GET " + String(url));
    Serial.println("GET " + String(url));
    client.print("?T1=");
    Serial.print("?T1=");
    client.print(T1);
    Serial.println(T1);
    client.print("&T2=");
    Serial.print("&T2=");
    client.print(T2);
    Serial.println(T2);
    client.print("&T3=");
    Serial.print("&T3=");
    client.print(T3);
    Serial.println(T3);
    client.print("&T4=");
    Serial.print("&T4=");
    client.print(T4);
    Serial.println(T4);
    client.print("&TP=");
    Serial.print("&TP=");
    client.print(TP);
    Serial.println(TP);
    client.print("&F3=");
    Serial.print("&F3=");
    client.print(F3);
    Serial.println(F3);
    client.print("&F4=");
    Serial.print("&F4=");
    client.print(F4);
    Serial.println(F4);
    client.print("&FX=");
    Serial.print("&FX=");
    client.print(FX);
    Serial.println(FX);
    client.print("&ZU=");
    Serial.print("&ZU=");
    client.print(ZU);
    Serial.println(ZU);
    client.print("&AB=");
    Serial.print("&AB=");
    client.print(AB);
    Serial.println(AB);
    client.print("&KO=");
    Serial.print("&KO=");
    client.print(String(KO));
    Serial.println(KO);
    client.print("&key=" + String(key));
    Serial.print("&key=" + String(key));
    client.println(" HTTP/1.1");
    Serial.println(" HTTP/1.1");
    client.print("Host: " + String(server));
    Serial.print("Host: " + String(server));
    client.println();
    Serial.println();
    client.println("User-Agent: Arduino");
    Serial.println("User-Agent: Arduino");
    client.println("Connection: close");
    Serial.println("Connection: close");
    client.println();
    Serial.println();
  }
  else
  {
    Serial.println(" ***** VERBINDUNG KANN NICHT HERGESTELLT WERDEN *****");
  }
}
// ***************************************************************************************** 
/*************************************
 * Funktion
 * Temperatur der Sensoren auslesen   *
 *
 *************************************/
void DS1820auslesen() 
{ 
  temp1 = getTemperature(sensor1);                      // Temperatur von DS1820 Sensor 1 ermitteln
  temp2 = getTemperature(sensor2);                      // Temperatur von DS1820 Sensor 2 ermitteln
  temp1 = temp1 + OS1;                                 // Offset hinzu
  temp2 = temp2 + OS2;
}

void DHT22lesen1()
{
  DHT1.read22(DHT1_PIN);
  dht1_h = DHT1.humidity, 1;                               //Luftfeuchte auslesen
  dht1_h = dht1_h + OSH3;                                  // Offset hinzu
  dht1_t = DHT1.temperature, 1;                            //Temperatur auslesen
  dht1_t = dht1_t + OS3;                                   // Offset hinzu
}

void DHT22lesen2()
{
  DHT2.read22(DHT2_PIN);
  dht2_h = DHT2.humidity, 1;                               //Luftfeuchte auslesen
  dht2_h = dht2_h + OSH4;                                  // Offset hinzu
  dht2_t = DHT2.temperature, 1;                            //Temperatur auslesen
  dht2_t = dht2_t + OS4;                                   // Offset hinzu
}
// ***************************************************************************************** 
void writeTimeToScratchpad(byte* address)
{
  ds.reset();
  ds.select(address);
  ds.write(0x44,1);
  delay(1000);
}

void readTimeFromScratchpad(byte* address, byte* data)
{
  ds.reset();
  ds.select(address);
  ds.write(0xBE);
  for (byte i=0;i<9;i++)
  {
    data[i] = ds.read();
  }
}

float getTemperature(byte* address)
{
  int tr;
  byte data[12];

  writeTimeToScratchpad(address);

  readTimeFromScratchpad(address,data);

  tr = data[0];

  if (data[1] > 0x80)
  {
    tr = !tr + 1; 
    tr = tr * -1; 
  }

  int cpc = data[7];
  int cr = data[6];

  tr = tr >> 1;

  return tr - (float)0.25 + (cpc - cr)/(float)cpc;
}
// ***************************************************************************************** 
void Automatik()
{
  Regelung(SollH, dht1_h, dht2_h, dht1_t, dht2_t, zuluft, abluft );  // Feuchteregelung
  boolean FanZeit = 0;
  if ((Stunde == 5 && Minute > 57) || (Stunde == 12 && Minute > 57 ) || (Stunde == 18 && Minute > 57) || (Stunde == 21 && Minute > 58) || (Stunde == 23 && Minute > 58 )) FanZeit = 1;  // Lüftungszeiten
  if (zuluft == 1 || FanZeit == 1)
  {
    digitalWrite(PinFanIN,LOW);  // Einschalten
    digitalWrite(PinFanOUT,LOW); // Einschalten
    kom = 1;   // Kommentar = 1   (Umluft EIN)
  } 
  else
  {
    digitalWrite(PinFanIN,HIGH); // Ausschalten
    digitalWrite(PinFanOUT,HIGH); // Ausschalten
    kom = 0;    // Kommentar = 0   (Umluft AUS)
  }   
}

void Handbetrieb()
{
  if (S2 == LOW)
  {
    digitalWrite(PinFanOUT,LOW); // Einschalten
    digitalWrite(PinFanIN,LOW); // Einschalten
    kom = 2;   // Kommentar = 2   (HAND EIN)
  } 
}
// ************************************** Feuchteregelung ********************************* 
void Regelung(float S_FI, float I_FI, float I_FA, float I_TI, float I_TA, boolean FAN_I, boolean FAN_A ) // S=SOll, I=IST, H=Feuchte, T= Temp, A=Aussen, I=Innen
{
  FAN_I = 0;
  FAN_A = 0;
  if ( S_FI - I_FI > 10.0 && I_FI < I_FA)  FAN_I = 1; // Feuchte zu niedrig
  if ( I_FI - S_FI > 10.0 && I_FI > I_FA)  FAN_I = 1; // Feuchte zu hoch
  if ( FAN_I == 1) FAN_A = 1;                         // Beide Lüfter einschalten
}
// ***************************************************************************************** 
void ZeitZaehler()
{
  Sekunde++; 
  if(Sekunde>=60)
  {
    Sekunde=0;
    Minute++;     
    if(Minute==60)
    {
      Minute=0;
      Stunde++;        
      if(Stunde==24)
      {
        Stunde=0;
      }
    }
  }    
}   
// ***************************************************************************************** 
void Monitor()
{
  Serial.print("Temperaturen:"); 
  Serial.print(temp1);
  Serial.print(" A: "); 
  Serial.println(temp2);
  Serial.print(" I: ");
  Serial.print(dht1_t);
  Serial.print(" TP: ");
  Serial.print(Taupunkt);
  Serial.print(" A: ");
  Serial.println(dht2_t);
  Serial.print("Luftfeuchten:"); 
  Serial.print(dht1_h);
  Serial.print(" S: ");
  Serial.print(SollH);
  Serial.print(" A: ");
  Serial.println(dht2_h);
  Serial.print("Schalter:");
  Serial.print(S1);
  Serial.print(" - ");
  Serial.println(S2);
  Serial.print(Stunde);
  Serial.print(":");
  Serial.print(Minute);
  Serial.print(":");
  Serial.println(Sekunde);
  Serial.print(hour());
  Serial.print(":");
  Serial.print(minute());
  Serial.print(":");
  Serial.println(second());
  Serial.println(komment[kom]);
}

// **********   NTP  ************************************************************
// Sende anfragen an NTP Time server
void NTPServer()
{
  sendNTPpacket(timeServer); 

    // Kurze Pause
  delay(1000);  
  if ( Udp.parsePacket() ) {  
    // Wenn Paket gelesen dann Daten auswerten
    Udp.read(packetBuffer,NTP_PACKET_SIZE);  // Paket in den Puffer schreiben

    //der Zeitstempel startet an byte 40 und ist 4 Byte bzw. 2 Wörter lang,
    // Als erstes die zwei Wörter extraieren:

    unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
    unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);  
    // Zwei Wörter in ein long integer wandeln
    // das ist die NTP Zeit (Sekunden seit 1.Jan 1900):
    unsigned long secsSince1900 = highWord << 16 | lowWord;  
    // Unix time startet am 1.Jan 1970. In Sekunden seit 1.Jan 1900 = 2208988800:
    const unsigned long seventyYears = 2208988800UL;     
    // 70 Jahre abziehen, Unix Zeit = epoch
    unsigned long epoch = secsSince1900 - seventyYears;                          
    UnixTime = epoch;
    Stunde = (epoch % 86400L) / 3600;  // Berechnung Stunde
    Stunde = Stunde + 1;               // GMT
    Minute = (epoch % 3600) / 60;      // Berechnung Minute
    Sekunde = (epoch %60);             // Berechnung Sekunde

    // Zeit an die seriellen Schnittstelle schreiben
    Serial.print("Die UTC Zeit ist ");       // UTC Greenwich Meridian (GMT)
    Serial.print(Stunde); // Stunde (86400 Sekunden pro Tag)
    Serial.print(':');  
    if ( (Minute) < 10 ) {
      // In den ersten 10 Minuten jeder Stunde eine führende '0'
      Serial.print('0');
    }
    Serial.print(Minute); // Minute (3600 Sekunden pro Stunde)
    Serial.print(':'); 
    if ( (Sekunde) < 10 ) {
      // In den ersten 10 Sekuden jeder Minute eine führende '0'
      Serial.print('0');
    }
    Serial.println(Sekunde); // Sekunde
  }
}


unsigned long sendNTPpacket(IPAddress& address)
{
  // Puffer Nullsetzen
  memset(packetBuffer, 0, NTP_PACKET_SIZE); 
  // Werte für NTP Request initialisieren
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49; 
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // Jetzt kann das Paket zur Zeitabfrage gesendet werden 		   
  Udp.beginPacket(address, 123); //NTP Abfrage über Port 123
  Udp.write(packetBuffer,NTP_PACKET_SIZE);
  Udp.endPacket(); 
}

void ZeitSync() 
{
  unsigned long pctime;
  const unsigned long DEFAULT_TIME = 1357041600; // 1. Jan 2013
  pctime = UnixTime;
  if( pctime >= DEFAULT_TIME)
  { // überprüfen ob Zeit ok ist (neuer als der 1. Jan 2013)
    setTime(pctime); // Synchronisiere Arduino Zeit mit der erhaltenen NTP Zeit
  }
}

























